==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/base/treeitem.cpp <==
// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005, 2008-2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treeitem.h"

#include <stdlib.h>
#include <cstring>
#include <iostream>

#include "treegroupbase.h"
#include "xmldefs.h"
#include "base_defs.h"
#include "links.h"
#include "treeview.h"
#include "basetools.h"
#include "../shared/textparser.h"
#include "../shared/lib_conversions.h"
#include "../shared/asciidoc.h"

TreeItem::TreeItem(TreeItem* prev, TreeGroupBase* parent)                 
{
  fMarked    = false;
  
  itemPrev   = prev;
  itemNext   = (TreeItem*)0;
  itemParent = parent;

  uID        = 0;
#if BASE_LNKCNT_IN_LIST == 0
  uLinkCnt   = 0;
#endif

  if (itemPrev != (TreeItem*)0)
    itemPrev->itemNext = this;
  if (itemParent != (TreeGroupBase*)0 && itemPrev == (TreeItem*)0)
    itemParent->itemFirstChild = this;
}


TreeItem::~TreeItem()
{
}

int  TreeItem::treeDepth()                                           
{
  int       nNum = 0;
  TreeGroupBase* item = itemParent;
  
  while (item != (TreeGroupBase*)0)
  {
    item = item->itemParent;
    nNum++;
  }
  
  return(nNum);
}


int  TreeItem::loadFromXML_all(SimpleXMLTransfer*    source,
                               LinkInfo*             links) 
{
  uID = source->attributeAsInt(XMLNAME_ID, 0);
  if (uID != 0)
  {
    links->newLinkableItem(this);
  }
    
  return(0);
}


int  TreeItem::saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  if (uID)
    dest->addAttribute(XMLNAME_SAVE_ID, uID);
  return(0);
}


void TreeItem::appendCommentIDs(std::vector<unsigned int>& ids)      
{
  TextParser           p;
  std::string          token;
  bool                 fNL;
  
  p.setText(getComment());
  
  while (p.hasToken())
  {
    p.getToken(token, fNL);
    if (token.find("<ID") == 0)
    {
      ids.push_back(atoi(token.substr(4, token.find('>')-4).c_str()));
    }    
  }
}


int  TreeItem::countCommentLinks(LinkInfo*    links,
                                 bool         fIncr)                 
{
  std::vector<unsigned int> l;
  TreeItem*                 dummy;
  int                       nRetCode = 0;
  
  appendCommentIDs(l);
  
  for (unsigned int n=0; n<l.size(); n++)
  {
    nRetCode |= links->countLinkCount(&dummy, l[n], TreeItem::ANY, fIncr);
  }
  
  return(nRetCode);
}

const char* TreeItem::getTypeString(TreeItem::eItemType type)         
{
  switch (type)
  {
   case GROUP:
    return("Group");
   case PIC:
    return("Picture");
   case ANY:
    return("Any");
   default:
    return("");
  }
}

void TreeItem::kw_add(std::set<std::string> kwset, bool fEvenIfHidden)
{
  if (!isHidden() || fEvenIfHidden)
  {
    std::string kw = "";
    std::set<std::string>::iterator it;
    
    // merge with current list of keywords in order to not create duplicates
    kw_set(kwset);
    
    it = kwset.begin();
    while (it != kwset.end())
    {
      if (kw.length())
        kw += ",";
      kw += (*it);
      it++;
    }  
    setKeywords(kw.c_str());
  }
}

void TreeItem::kw_set(std::set<std::string>& kwset)
{
  if (!isHidden())
  {
    std::set<std::string> mykwset = BaseTools::KeywordList_Set(getKeywords());

    // merge my keywords into kwset
    kwset.insert(mykwset.begin(), mykwset.end());
  }
}

void TreeItem::kw_list(std::vector<std::string>& kwlist)
{
  if (!isHidden())
  {
    std::string            kw    = getKeywords();
    std::string::size_type pos   = kw.find(',');
    std::string::size_type start = 0;

    while (pos != std::string::npos)
    {
      if (pos > start)
        kwlist.push_back(kw.substr(start, pos-start));
      start = pos+1;
      pos   = kw.find(',', start);
    }
    if (start == 0 && kw.length())
      kwlist.push_back(kw);
    else if (start < kw.length())
      kwlist.push_back(kw.substr(start, kw.length()-start));
  }
}

void TreeItem::kw_replace(std::string from, std::string to, int mode)
{
  // todo: prevent duplicate keywords
  //std::cout << "kw_replace: " << getText() << "\n";
  if (!isHidden())
  {
    std::string kwnew = "";
    std::vector<std::string> kwlist;
    kw_list(kwlist);
    std::vector<std::string>::iterator i = kwlist.begin();
    bool fChanged = false;
    
    while (i != kwlist.end())
    {
      if (mode == 1 && (*i).compare(from) == 0)   // whole word
      {
        kwnew += "," + to;
        fChanged = true;
      }
      else if (mode == 0 && (*i).find(from) == 0) // beginning is sufficient
      {
        kwnew += "," + (*i).replace(0, from.length(), to);
        fChanged = true;        
      }
      else
        kwnew += "," + (*i);
      i++;
    }
    if (fChanged)
    {
      if (kwnew[0] == ',')
        kwnew = kwnew.substr(1);
      //std::cout << "Keywords: " << getKeywords() << " -> " << kwnew << "\n";
      setKeywords(trim(kwnew).c_str());
    }
  }
}

bool TreeItem::isHidden()
{
  return(false);
}

void TreeItem::RecursiveList(std::set<TreeItem*>& set)
{
  set.insert(this);
}

std::string TreeItem::getCommentHtml()
{
  std::string strc = getComment();
  if (strc.find("<a href") != std::string::npos ||
      strc.find("<table") != std::string::npos ||
      strc.find("<ul>") != std::string::npos)
    return(strc);
  else
  {
    // asciidoc is assumed
    return(AsciiDoc::ToHtml(strc));
  }
}

std::string TreeItem::ConvertComment(std::string in, int nNew)
{
  std::string out = in;
  
  int nOld;
  if (itemParent == 0)
  {
    nOld = (dynamic_cast<TreeGroupRoot*>(this))->GetVersionInfo();
  }
  else
  {
    TreeGroupBase* parent = itemParent;
    while (parent->itemParent)
      parent = parent->itemParent;
    nOld = (dynamic_cast<TreeGroupRoot*>(parent))->GetVersionInfo();
  }
  
  
  if (nOld == 2 && nOld != nNew)
  {
    out = "";
    bool fConv = false;
    for (std::string::size_type n=0; n<in.length(); n++)
    {
      // todo: das könnte man erheblich schlauer machen, damit es nicht mehr als zwei
      //       aufeinanderfolgenden newlines gibt. Außerdem sollte whitespace dazwischen
      //       ignoriert werden.
      if (in[n] == '\n' && n+1 != in.length())
      {
        out += "\n\n";
        fConv = true;
      }
      else
        out += in[n];
    }
    
    if (false && fConv)
    {
      std::cout << "\n----------------------------------------\n" << in << "\n";
    }
    
    // 
    nOld = 3;
  }
  
// template:  if (nOld == 3 && nOld != nNew)
// template:  {
// template:    // todo: convert
// template:    nOld = 4;
// template:  }
      
  return out;
}

==> fotoalbum-0.104.tar.bz2: fotoalbum-0.104/src/base/treeitem.cpp <==
// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005, 2008-2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treeitem.h"

#include <stdlib.h>
#include <cstring>
#include <iostream>

#include "treegroupbase.h"
#include "xmldefs.h"
#include "base_defs.h"
#include "links.h"
#include "treeview.h"
#include "basetools.h"
#include "../shared/textparser.h"
#include "../shared/lib_conversions.h"
#include "../shared/asciidoc.h"
#include "../shared/utf8.h"

TreeItem::TreeItem(TreeItem* prev, TreeGroupBase* parent)                 
{
  fMarked    = false;
  
  itemPrev   = prev;
  itemNext   = (TreeItem*)0;
  itemParent = parent;

  uID        = 0;
#if BASE_LNKCNT_IN_LIST == 0
  uLinkCnt   = 0;
#endif

  if (itemPrev != (TreeItem*)0)
    itemPrev->itemNext = this;
  if (itemParent != (TreeGroupBase*)0 && itemPrev == (TreeItem*)0)
    itemParent->itemFirstChild = this;
}


TreeItem::~TreeItem()
{
}

int  TreeItem::treeDepth()                                           
{
  int       nNum = 0;
  TreeGroupBase* item = itemParent;
  
  while (item != (TreeGroupBase*)0)
  {
    item = item->itemParent;
    nNum++;
  }
  
  return(nNum);
}


int  TreeItem::loadFromXML_all(SimpleXMLTransfer*    source,
                               LinkInfo*             links) 
{
  uID = source->attributeAsInt(XMLNAME_ID, 0);
  if (uID != 0)
  {
    links->newLinkableItem(this);
  }
    
  return(0);
}


int  TreeItem::saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  if (uID)
    dest->addAttribute(XMLNAME_SAVE_ID, uID);
  return(0);
}


void TreeItem::appendCommentIDs(std::vector<unsigned int>& ids)      
{
  TextParser           p;
  std::string          token;
  bool                 fNL;
  
  p.setText(getComment());
  
  while (p.hasToken())
  {
    p.getToken(token, fNL);
    if (token.find("<ID") == 0)
    {
      ids.push_back(atoi(token.substr(4, token.find('>')-4).c_str()));
    }    
  }
}


int  TreeItem::countCommentLinks(LinkInfo*    links,
                                 bool         fIncr)                 
{
  std::vector<unsigned int> l;
  TreeItem*                 dummy;
  int                       nRetCode = 0;
  
  appendCommentIDs(l);
  
  for (unsigned int n=0; n<l.size(); n++)
  {
    nRetCode |= links->countLinkCount(&dummy, l[n], TreeItem::ANY, fIncr);
  }
  
  return(nRetCode);
}

const char* TreeItem::getTypeString(TreeItem::eItemType type)         
{
  switch (type)
  {
   case GROUP:
    return("Group");
   case PIC:
    return("Picture");
   case ANY:
    return("Any");
   default:
    return("");
  }
}

void TreeItem::kw_add(std::set<std::string> kwset, bool fEvenIfHidden)
{
  if (!isHidden() || fEvenIfHidden)
  {
    std::string kw = "";
    std::set<std::string>::iterator it;
    
    // merge with current list of keywords in order to not create duplicates
    kw_set(kwset);
    
    it = kwset.begin();
    while (it != kwset.end())
    {
      if (kw.length())
        kw += ",";
      kw += (*it);
      it++;
    }  
    setKeywords(kw.c_str());
  }
}

void TreeItem::kw_set(std::set<std::string>& kwset)
{
  if (!isHidden())
  {
    std::set<std::string> mykwset = BaseTools::KeywordList_Set(getKeywords());

    // merge my keywords into kwset
    kwset.insert(mykwset.begin(), mykwset.end());
  }
}

void TreeItem::kw_list(std::vector<std::string>& kwlist)
{
  if (!isHidden())
  {
    std::string            kw    = getKeywords();
    std::string::size_type pos   = kw.find(',');
    std::string::size_type start = 0;

    while (pos != std::string::npos)
    {
      if (pos > start)
        kwlist.push_back(kw.substr(start, pos-start));
      start = pos+1;
      pos   = kw.find(',', start);
    }
    if (start == 0 && kw.length())
      kwlist.push_back(kw);
    else if (start < kw.length())
      kwlist.push_back(kw.substr(start, kw.length()-start));
  }
}

void TreeItem::kw_replace(std::string from, std::string to, int mode)
{
  // todo: prevent duplicate keywords
  //std::cout << "kw_replace: " << getText() << "\n";
  if (!isHidden())
  {
    std::string kwnew = "";
    std::vector<std::string> kwlist;
    kw_list(kwlist);
    std::vector<std::string>::iterator i = kwlist.begin();
    bool fChanged = false;
    
    while (i != kwlist.end())
    {
      if (mode == 1 && (*i).compare(from) == 0)   // whole word
      {
        kwnew += "," + to;
        fChanged = true;
      }
      else if (mode == 0 && (*i).find(from) == 0) // beginning is sufficient
      {
        kwnew += "," + (*i).replace(0, from.length(), to);
        fChanged = true;        
      }
      else
        kwnew += "," + (*i);
      i++;
    }
    if (fChanged)
    {
      if (kwnew[0] == ',')
        kwnew = kwnew.substr(1);
      //std::cout << "Keywords: " << getKeywords() << " -> " << kwnew << "\n";
      setKeywords(UTF8::trim(kwnew).c_str());
    }
  }
}

bool TreeItem::isHidden()
{
  return(false);
}

void TreeItem::RecursiveList(std::set<TreeItem*>& set)
{
  set.insert(this);
}

std::string TreeItem::getCommentHtml()
{
  std::string strc = getComment();
  if (strc.find("<a href") != std::string::npos ||
      strc.find("<table") != std::string::npos ||
      strc.find("<ul>") != std::string::npos)
    return(strc);
  else
  {
    // asciidoc is assumed
    return(AsciiDoc::ToHtml(strc));
  }
}

std::string TreeItem::ConvertComment(std::string in, int nNew)
{
  std::string out = in;
  
  int nOld;
  if (itemParent == 0)
  {
    nOld = (dynamic_cast<TreeGroupRoot*>(this))->GetVersionInfo();
  }
  else
  {
    TreeGroupBase* parent = itemParent;
    while (parent->itemParent)
      parent = parent->itemParent;
    nOld = (dynamic_cast<TreeGroupRoot*>(parent))->GetVersionInfo();
  }
  
  
  if (nOld == 2 && nOld != nNew)
  {
    out = "";
    bool fConv = false;
    for (std::string::size_type n=0; n<in.length(); n++)
    {
      // todo: das könnte man erheblich schlauer machen, damit es nicht mehr als zwei
      //       aufeinanderfolgenden newlines gibt. Außerdem sollte whitespace dazwischen
      //       ignoriert werden.
      if (in[n] == '\n' && n+1 != in.length())
      {
        out += "\n\n";
        fConv = true;
      }
      else
        out += in[n];
    }
    
    if (false && fConv)
    {
      std::cout << "\n----------------------------------------\n" << in << "\n";
    }
    
    // 
    nOld = 3;
  }
  
  if (nOld == 3 && nOld != nNew)
  {
    // no conversion necessary
    nOld = 4;
  }
  
  
// template:  if (nOld == 4 && nOld != nNew)
// template:  {
// template:    // todo: convert
// template:    nOld = 5;
// template:  }

  return out;
}
